/*****************************************************************************/
/*                                                                           */
/*	 5110Comm.c                                                              */
/*                                                                           */
/*   Win32 terminal program to read data from TSC 5110A into file.           */
/*   Uses dialog box as its main window.                                     */
/*   Has no menu, toolbuttons or other windows.                              */
/*                                                                           */
/*   Major revision record:                                                  */
/*      09/20/00    Created                                                  */
/*		09/22/00	Running                                                  */
/*		08/06/03	Added Stable32 launch button                             */
/*                                                                           */
/*	 Version Log:                                                            */
/*      09/22/00	Version 1.00 released                                    */
/*		10/01/00	Version 1.01 with enhanced file opening                  */
/*		08/06/03	Version 1.02 with Stable32 launching                     */
/*		09/04/03	Version 1.03 with quoted filenames for Stable32 launch   */
/*		12/08/05	Version 1.04 with file mapping removed                   */
/*		02/12/07	Version 1.05 with HTML help                              */
/*                                                                           */
/*   (c) 2000-7  W. Riley Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

// Includes
#include "5110Comm.h"
#include "resource.h"

// Global variables
// Note: All string allocations have extra character for EOS
BOOL bRead;									// Flag to read comm data
HWND hwndMain;								// Main window handle
HINSTANCE hInst;							// Application instance
HICON hIcon;								// Application icon
HANDLE hAccel;                              // Handle for accelerators
char szTrace[BUFFER_SIZE+1];				// Trace buffer

// WinMain() function
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	PSTR szCmdLine, int nCmdShow)
{
	// Local variables
    MSG msg;								// Main message loop message

	// Get handle to application icon
	hIcon=LoadIcon(hInstance, "CommIcon");

	// Set global instance handle
	hInst=hInstance;

	// Create dialog box to be used as main window
	hwndMain=CreateDialog(hInstance, "CommBox", NULL, CommDlgProc);
	
    // Load accererators
    hAccel=LoadAccelerators(hInstance, (LPCTSTR)IDR_ACCEL);

	// Display main window
    ShowWindow(hwndMain, nCmdShow);
    UpdateWindow(hwndMain);

	// Set the application system menu icon
	SetClassLong(hwndMain, GCL_HICON, (LONG)hIcon);

   // Main message loop
    while(GetMessage(&msg, NULL, 0, 0))
    {
        // Is this an accelerator message?
        // F1 will invoke help
        // Esc will also close program, but is not accelerator
        // Also right & left arrows will move up & down thru buttons
        // and spacebar will press them
        // And, of course, there are Alt-letter button shortcut keys 
        if(!TranslateAccelerator(hwndMain, hAccel, &msg))
        {
            // Is this a dialog box message?
            if(IsDialogMessage(hwndMain, &msg))
            {
                // If so, skip normal message handling
                continue;
            }
        
            // Otherwise process message as normal
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
    return msg.wParam;
}

/*****************************************************************************/
/*                                                                           */
/*							YieldControl()                                   */
/*                                                                           */
/*		Function to allow Windows to take control & service other messages   */
/*                                                                           */
/*      Parameters:                                                          */
/*      	None 	(void)                                                   */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			09/20/00	Added from Stable32                                  */
/*                                                                           */
/*****************************************************************************/
void WINAPI YieldControl(void)
{
	// Local variables
	MSG msg;
	
	// Give stop button the focus
	SetFocus(GetDlgItem(hwndMain, IDC_COMM_START));

	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if(!IsDialogMessage(hwndMain, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
}

/*****************************************************************************/
/*                                                                           */
/*							ShortenExp()                                     */
/*                                                                           */
/*		Function to shorten all 3-digit exponents in a string                */
/*      Use with Microsoft Visual C++ 5.0 for legacy display layouts         */
/*      Changes all numeric value formats of passed numeric value string     */
/*		from 3 digits to 2 digits                                            */
/*                                                                           */
/*      Parameters:                                                          */
/*      	szString		Numeric value string                             */
/*                                                                           */
/*      Return:                                                              */
/*			None	(void)                                                   */
/*                                                                           */
/*		Revision record:                                                     */
/*			09/216/00	Created from same function in Stable32               */
/*                                                                           */
/*****************************************************************************/
void WINAPI ShortenExp( char *szString)
{
	// Local variables
    int i;		// Main index
	int j;		// Secondary index
	int len;	// String length

	// Find string length
	len=strlen(szString);
	
	// Scan for next 'e' in the string followed by a + or - and then a zero
	// Looking for pattern e+0dd or e-0dd
	// Scan stops 4 chars before end of string
	for(i=0; i<len-4; i++)
	{
		if( (szString[i]=='e') && (szString[i+1]=='+' || szString[i+1]=='-') &&
			(szString[i+2]=='0') )
		{
			// Original format is e+0dd or e-0dd
			// Want to eliminate the leading exp digit (always zero)
			// Modified format will be e+dd or e-dd
			// Shift up the string chars (and EOS) after the unwanted 0
			for(j=i+2; j<len; j++)
			{
				szString[j]=szString[j+1];
			}
			len--;
		}
	}

}

/*****************************************************************************/
/*                                                                           */
/*							   CheckFilename()                               */
/*                                                                           */
/*		Function to check whether string is valid filename                   */
/*                                                                           */
/*      Accept szFile.  Test if a valid filename (with or w/o path).         */
/*      If (optional) path NG, display message box in hWnd offering to       */
/*      create it (using label passed by szLabel).  Create new path if       */
/*		requested.  Return TRUE for OK filename (existing or new) or FALSE   */
/*      if filename NG.  Also return new path, name & ext.                   */
/*                                                                           */
/*      Parameters:                                                          */
/*      	szFile		File name string                                     */
/*      	szTitle		File title string                                    */
/*      	szPath		File path string                                     */
/*      	hWnd		Parent window handle                                 */
/*      	szLabel		Label string for message box title                                        */
/*                                                                           */
/*      Return:                                                              */
/*			BOOL		Path status                                          */
/*                                                                           */
/*		Revision record:                                                     */
/*			09/20/00	Created from same function in Stable32               */
/*                                                                           */
/*****************************************************************************/
BOOL WINAPI CheckFilename(char *szFile, char *szTitle, char *szPath, 
	char *szName, char *szExt, HWND hWnd, char *szLabel)
{
	// Local variables
	char szBuffer[BUFFER_SIZE+1];		// General buffer
	char szCWD[FILENAME_SIZE+1];		// Current working directory
	char szDrive[BUFFER_SIZE+1];		// Drive buffer
	char szDirectory[FILENAME_SIZE+1];	// Directory buffer
	BOOL bFile=TRUE;					// Filename status flag

	// Split filename into its parts
	_splitpath(szFile, szDrive, szDirectory, szName, szExt);

	
	// If drive is not NULL, make sure directory starts with a backslash
	if(strlen(szDrive))
	{
		if(szDirectory[0]!='\\')
		{
			strcpy(szBuffer, "\\");
			strcat(szBuffer, szDirectory);
			strcpy(szDirectory, szBuffer);
		}
	}

	// Reassemble full filename in standard format
	_makepath(szFile, szDrive, szDirectory, szName, szExt);

	// Reassemble path
	strcpy(szPath, szDrive);
	strcat(szPath, szDirectory);

	// Reassemble file title
	strcpy(szTitle, szName);
	strcat(szTitle, szExt);

	// Check if szTitle is a valid file title (name + ext)
	// A filename cannot be blank
	// Windows 95/NT filenames can contain any characters except:
	// ? " / \ < > * | :
	// Note that in a string constant the " and \ must be denoted
	// by their excape codes \" and \\ respectively

	if( (strlen(szTitle)==0) ||
		(strcspn(szTitle, "?\"/\\<>*|:") < strlen(szTitle)) )
	{
		// Display error message
		MessageBox(hWnd, "Invalid Filename", szLabel, MB_ICONEXCLAMATION);

		// Return FALSE if couldn't open file
		return FALSE;
	}

	// Check path if not null (backslash only)
	if(strlen(szPath)>1)
	{
		// Save current working directory
		_getcwd(szCWD, FILENAME_SIZE);

		// Check if path exists
		if(_chdir(szPath)!=0)
		{
			// Display error message
			strcpy(szBuffer,
				"The specified path does not exits.\nDo you wish to create it?");

			// Respond to user's yes/no choice
			if(MessageBox(hWnd, szBuffer, szLabel, MB_YESNO | MB_ICONQUESTION)==
				IDYES)
			{
				// Create new path
				_mkdir(szPath);

				// Path now exists
				bFile=TRUE;
			}
			else // Don't create new directory
			{
				// Path doesn't exist
				bFile=FALSE;
			}
		}
		else // Path already exists
		{
			bFile=TRUE;
		}

		// Restore CWD
		_chdir(szCWD);
	}

	// Return filename status
	return bFile;
}

/*****************************************************************************/
/*                                                                           */
/*                           DrawBitmap()                                    */
/*                                                                           */
/* 	Function to draw a bitmap on a display surface.                          */
/*                                                                           */
/*  See Petzold, "Programming Windows 3.1", p.631                            */
/*                                                                           */
/*****************************************************************************/

void WINAPI DrawBitmap(HDC hdc, HBITMAP hBitmap, short xStart,
	short yStart)
{
	BITMAP bm;
	HDC hdcMemory;
	POINT ptSize, ptOrg;

	hdcMemory=CreateCompatibleDC(hdc);
	SelectObject(hdcMemory, hBitmap);
	SetMapMode(hdcMemory, GetMapMode(hdc));

	GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
	ptSize.x=bm.bmWidth;
	ptSize.y=bm.bmHeight;
	DPtoLP(hdc, &ptSize, 1);

	ptOrg.x=0;
	ptOrg.y=0;
	DPtoLP(hdcMemory, &ptOrg, 1);

	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y,
		hdcMemory, ptOrg.x, ptOrg.y, SRCCOPY);

	DeleteDC(hdcMemory);
}

/*****************************************************************************/
