/*****************************************************************************/
/*                                                                           */
/*   5110Comm2.c                                                             */
/*                                                                           */
/*   Dialog box procedure for 5110Comm program                               */
/*                                                                           */
/*   (c) 2000-5  W. Riley Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

// Includes
#include "5110Comm.h"
#include "resource.h"

/*****************************************************************************/
/*                                                                           */
/*                   Comm Dialog Procedure                                   */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/20/00    Created                                              */
/*			09/21/00	Running                                              */
/*			09/22/00	Seems AOK                                            */
/*			09/23/00	A few more improvements                              */
/*			09/25/00	Added plotting                                       */
/*			09/26/00	Cleaned up a few things                              */
/*			09/27/00	Ditto - BoundsChecker clean                          */
/*						Expanded plot size                                   */
/*			09/28/00	Added list and plot sizes to ini file                */
/*						Switched to dynamic memory allocation for plot data  */
/*						Changed red pen creation and deletion                */
/*						Added green pen and line at average for freq plot    */
/*			09/29/00	Do plotting to bitmap in memory to reduce flicker    */
/*			09/30/00	Added size edit box & realloc of plot array sizes    */
/*						Added display of last frequency                      */
/*						Improved appearance of phase and freq plots          */
/*			10/01/00	Shortened avg freq line                              */
/*			02/18/02	Enhanced file opening choices                        */
/*			08/06/03	Added button to invoke Stable32                      */
/*			09/04/03	Added quotes around filenames when Stable32 invoked  */
/*			12/08/05	File mapping removed - caused error msg to appear    */
/*                                                                           */
/*****************************************************************************/

BOOL CALLBACK CommDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    // Macros
    #define NUM_COMMPORTS 4				// # COM ports
	#define PLOT_SIZE 240				// Default # plot & list points
	#define X_DIVS 8.0					// # x-axis plot divisions
	#define Y_DIVS 5.0					// # y-axis plot divisions
    #define PORTNAME_SIZE 6

	#if(0) // New code 12/26/00 - Removed 12/08/05
	// File mapping name
	#define MAP_NAME	"MyFileMappingObject"

	// File mapping variables
	HANDLE hMapFile=NULL;				// Handle of file mapping object
	LPSTR lpMapAddress;					// Pointer to mapped file data
	#endif 

    // Local variables
    // Ints
    static int i;                       // General index
    static int nDummy;                  // Dummy var to avoid complier warnings
    static int nPrev;                   // Previous value of an entry
    static int nEntry;                  // Entry value to be validated
    static int nComm=1;                 // Active COM port # (1-4)
	static int nNum=0;					// # data points read
	static int nSize;					// Plot and list size
    // BOOLs
    static BOOL bOpen=FALSE;            // Comm open flag
    static BOOL bErrorTrap=FALSE;		// Error trap flag
    // Char arrays
    static char szData[BUFFER_SIZE+1];			// Data buffer
    static char szBuffer[2*FILENAME_SIZE+1];	// General text buffer
												// Made large because it is used
												// to hold two filenames
    static char szText[BUFFER_SIZE+1];			// Text buffer
    static char szPort[PORTNAME_SIZE+1];		// Comm port name buffer
    // Filename buffers
	static char szTitle[FILENAME_SIZE+1];       // Title buffer
    static char szName[FILENAME_SIZE+1];		// Name buffer
    static char szExt[FILENAME_SIZE+1];			// Extension buffer
	static char szHelpFile[FILENAME_SIZE+1];	// Full path & filename to help
	static char szIniFile[FILENAME_SIZE+1];		// Full path & filename to ini
	static char szDataFile[FILENAME_SIZE+1];	// Full path & filename to data
	static char szStablePath[FILENAME_SIZE+1];  // Full path & filename to Stable32
	// Drive & path variables
	static char szDrive[BUFFER_SIZE+1];			// DOS drive name where program started
	static char szDirectory[BUFFER_SIZE+1];		// DOS directory where program started
	static char szPath[FILENAME_SIZE+1];		// Path where program started
	// Floats
	static double fScale;				// Scale factor
	static double fData;				// Data value
	static double fPrevData;			// previous data value
	static double fPrev;				// Previous value
	static double fAvg;					// Average freq
	static double fDev;					// Std dev of freq
    // Comm Params
    static DCB dcb;                     // Device control block structure
    static COMMTIMEOUTS to;             // Communications timeouts structure
    static HANDLE hComm;                // Handle to comm port
    static BOOL bSuccess;               // Return value from comm function
    static DWORD dwNumBytes;            // # bytes read
	// File params
	static HANDLE hFile;				// Handle to disk file
	// Menu params
	static HANDLE hMenu;				// Handle to system menu
	// Handles
	static HPEN hRedPen;				// Handle to red pen
	static HPEN hGreenPen;				// Handle to green pen
	// Plot variables
	static HWND hPlot;					// Handle to plot window
	static HDC hdcPlot;					// Handle to plot device context
    static RECT rect;					// Plot rectangle
	static BOOL bPlot=FALSE;			// Flag to show plot
	static BOOL bFreq=FALSE;			// Data type: 0=Phase, 1=Freq
	static double fPPoint;				// Current phase plot point
	static double fFPoint;				// Current freq plot point
    static double *fPPlot;				// Pointer to phase plot data array
    static double *fFPlot;				// Pointer to freq plot data array
	static int nPlot=0;					// Plot index
	static int nLastY;					// Last y-axis index for freq plotting
	static nDiv;						// # sec/div on x-axis
	static double fPMin, fPMax;			// Max & min phase data values
	static double fFMin, fFMax;			// Max & min freq data values
    static double fRange;               // Plot range
	static double fXScale, fYScale;     // Plot scale factors
	// New memory plotting variables
	static HANDLE hBitmap;				// Bitmap handle
	static HDC hdcMem;					// Handle to memory device context
	static HDC hdcBitmap;				// Handle to bitmap device context

    // Start of code
    // Dialog box message switch statement
    switch(msg)
    {
        case WM_INITDIALOG:
        {
			// Get working directory from arg[0]
			// This is better than using the current working directory
			// because it is always the full path where program started
			// regardless of how the program's working directory is set up
			// or what directory is currently active on the computer

			_splitpath(__argv[0], szDrive, szDirectory, NULL, NULL);
			strcpy(szPath, szDrive);
			strcat(szPath, szDirectory);

			// Set up the full ini file path & name
			strcpy(szIniFile, szPath);
			strcat(szIniFile, "5110Comm.ini");

			// Set up the full help file path & name
			strcpy(szHelpFile, szPath);
			strcat(szHelpFile, "5110Comm.hlp");

			// Read previous data filename from ini file
			GetPrivateProfileString("Preferences", "File", "",
				szDataFile,	BUFFER_SIZE, szIniFile);

			// Read Stable32 path & filename from ini file
			GetPrivateProfileString("Preferences", "Stable", "",
				szStablePath,	BUFFER_SIZE, szIniFile);

			// Set up the default full data file path & name
			// if not read from ini file
			if(strlen(szDataFile)==0)
			{
				strcpy(szDataFile, szPath);
				strcat(szDataFile, "5110Comm.dat");
			}

			// Get handle and device context to plot area
            hPlot=GETHCTL(IDC_COMM_PLOTAREA);
            hdcPlot=GetDC(hPlot);

			// Create red & green pens
			// Note: Create pens here and keep them for the duration
			// of the program rather than creating and deleting them
			// every time the plot is updated
			hRedPen=CreatePen(PS_SOLID, 1, 0x000000FF);
			hGreenPen=CreatePen(PS_SOLID, 1, 0x0000FF00);

			// Read previous COM port from ini file
			nComm=GetPrivateProfileInt("Preferences", "Comm", 1,
				szIniFile);
            
			// Read previous scale factor from ini file
			GetPrivateProfileString("Preferences", "Scale", "1.0", szBuffer,
				BUFFER_SIZE, szIniFile);
			fScale=atof(szBuffer);

			// Read previous plot flag from ini file
			bPlot=GetPrivateProfileInt("Preferences", "Plot", 0,
				szIniFile);

			// Read previous freq flag from ini file
			bFreq=GetPrivateProfileInt("Preferences", "Freq", 0,
				szIniFile);

			// Read plot & list size from ini file
			nSize=GetPrivateProfileInt("Preferences", "Size", PLOT_SIZE,
				szIniFile);

			// Make sure size is positive, at least 8, and a multiple of 8
			nSize=abs(nSize);
			if(nSize<8)
			{
				nSize=8;
			}
			nSize=nSize+(8-(nSize%8)-8);
			
			// Display size
			sprintf(szBuffer, "%d", nSize);
            SetWindowText(GETHCTL(IDC_COMM_SIZE), szBuffer);

			// Allocate memory for plot data arrays
			if((fPPlot=(double*)malloc((nSize+2)*sizeof(F_TYPE)))==NULL)
			{
				// Error message
                MessageBox(hDlg, "Unable to allocate memory for plot array.",
					"5110Comm", MB_ICONEXCLAMATION);
			}
			if((fFPlot=(double*)malloc((nSize+2)*sizeof(F_TYPE)))==NULL)
			{
				// Error message
                MessageBox(hDlg, "Unable to allocate memory for plot array.",
					"5110Comm", MB_ICONEXCLAMATION);
			}

			// Set plot and list button states
            Button_SetCheck(GETHCTL(IDC_COMM_PLOTFREQ), (bPlot && bFreq));
            Button_SetCheck(GETHCTL(IDC_COMM_PLOTPHASE), (bPlot && !bFreq));
            Button_SetCheck(GETHCTL(IDC_COMM_LIST), !bPlot);

			// Clear terminal text list box
            ListBox_ResetContent(GETHCTL(IDC_COMM_TEXT));

            // Reset read flag - it will be toggled to set it
			bRead=FALSE;
			
			// Check for valid filename
			// Input is szDataFile to validate
			// Returns TRUE if filename OK, otherwise FALSE
			// Displays messages if filename invalid or if path
			// doesn't exist (offers to create)
			// Also returns file title, path, name and extension
			CheckFilename(szDataFile, szTitle, szPath, szName, szExt, hDlg,
				"TSC 5110A Comm");

			// Load default data filename
			SetWindowText(GETHCTL(IDC_COMM_FILENAME), szDataFile);

			// Fill up the COM # combo box list with COM1-COM4
            // Note: Index is nComm-1
            for(i=0; i<NUM_COMMPORTS; i++)
            {
                sprintf(szBuffer, "COM%d", i+1);
                nDummy=ComboBox_InsertString(GETHCTL(IDC_COMM_COMM), i,
                    szBuffer);
            }

            // Show current COM # - Index is nComm-1
            nDummy=ComboBox_SetCurSel(GETHCTL(IDC_COMM_COMM), nComm-1);

            // Save comm port name
            GetWindowText(GETHCTL(IDC_COMM_COMM), szPort,
				BUFFER_SIZE);

			// Initialize the comm open status indicator state
            Button_SetCheck(GETHCTL(IDC_COMM_OPEN), bOpen);

             // Display scale factor
            sprintf(szBuffer, "%+17.10e", fScale);
			ShortenExp(szBuffer);
            SetWindowText(GETHCTL(IDC_COMM_SCALE), szBuffer);

			// Copy welcoming message into buffer
			strcpy(szBuffer, "TSC 5110A Comm Program");

			// Display message
			SendMessage(hDlg, WM_COMMAND,
				GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));

			if(bPlot)
			{	
				ShowWindow(GETHCTL(IDC_COMM_PLOTAREA), SW_SHOWNORMAL);
				ShowWindow(GETHCTL(IDC_COMM_TEXT), SW_HIDE);
			}
			else // Text
			{	
				ShowWindow(GETHCTL(IDC_COMM_PLOTAREA), SW_HIDE);
				ShowWindow(GETHCTL(IDC_COMM_TEXT), SW_SHOWNORMAL);
			}
			
			// Try to open the COM port read from the ini file
            SendMessage(hDlg, WM_COMMAND,
                GET_WM_COMMAND_MPS(IDC_COMM_OPENPORT, NULL, 0));
            
			// Disable the start button if com port isn't open
			EnableWindow(GETHCTL(IDC_COMM_START), bOpen);
            
			// Display message if weren't able to open COM port 
            if(!bOpen)
            {
				// Copy warning message into buffer
				strcpy(szBuffer, "Please open a comm port");

				// Display message
				SendMessage(hDlg, WM_COMMAND,
					GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));

				#if(0) // Trial code 12/26/00 - Removed 12/08/05
				// Try file mapping to send data to Stable32
				hMapFile=OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, MAP_NAME);
				if(hMapFile)
				{
					lpMapAddress=MapViewOfFile(hMapFile, FILE_MAP_ALL_ACCESS,
						0, 0, 0);
					strcpy(lpMapAddress, "Hello from 5110Comm");
				}
				else
				{
					TRACE_STR("OpenFileMapping Failed");
				}
				#endif
            }

            // Initialization done
            return TRUE;
        }

		case WM_SYSCOMMAND:
		{
			// Always respond to minimize and restore messsages
			if( (wParam==SC_MINIMIZE) || (wParam==SC_RESTORE) )
			{
				break;
			}

			// Intercept and ignore other system messages while reading data
			if(bRead)
			{
				return TRUE;
			}
		}
		break;
		
		case WM_COMMAND:
        {
            switch(GET_WM_COMMAND_ID(wParam, lParam))
            {
                // Respond to help button
                case IDC_COMM_HELP:
                {
                    // Invoke help
					#if(0) // Obsolete WinHelp
					WinHelp(hDlg, szHelpFile, HELP_CONTENTS, 0L);
					#endif
					#if(1) // New HTML help 02/12/07
					HtmlHelp(hDlg, "5110Comm.chm::INDEX.HTM", HH_DISPLAY_TOPIC,
                        0L);
					#endif
                    break;
                }

                // Respond to Stable32 button
                case IDC_COMM_STABLE:
                {
					// Compose command
					// Note: Filenames are enclosed in quotes to handle
					// names with embedded spaces
					// Path to Stable32 in manually put into ini file
					strcpy(szBuffer, "\"");
					strcat(szBuffer, szStablePath);
					strcat(szBuffer, "\"");
					// Tau is always 1 second, data is always phase
					// Data scaling is already done
					strcat(szBuffer, " -t 1.0 -p \"");
					// Data filename is entered by user on screen
					strcat(szBuffer, szDataFile);
					// Execute Auto1 script, skip opening screens
					strcat(szBuffer, "\" -o auto1 -o skip");

					// Invoke Stable32
                    WinExec(szBuffer, SW_SHOWNORMAL);
                    break;
                }

                // Respond to start button
                case IDC_COMM_START:
                {
					// Toggle read flag
					bRead=!bRead;

					// Configure operation
					if(bRead)
					{
						// Zero the # of points
						nNum=0;
						nPlot=0;
						
						// Open the disk file
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_COMM_OPENFILE, NULL, 0));

						// Skip rest of file open cancelled
						if(!bRead)
						{
							break;
						}
 
						// Change the start button to stop
						SetWindowText(GETHCTL(IDC_COMM_START), "&Stop");

						// Change the prompt text
						SetWindowText(GETHCTL(IDC_COMM_PROMPT),
							"Press Stop to end run.");

						// Clear the plot labels
						SetWindowText(GETHCTL(IDC_COMM_PLOTSCALE), "");
						SetWindowText(GETHCTL(IDC_COMM_AVG), "");
					}
					else
					{
                        // Close the data file
						CloseHandle(hFile);
						
						// Change the stop button to start
						SetWindowText(GETHCTL(IDC_COMM_START), "&Start");

						// Change the prompt text
						SetWindowText(GETHCTL(IDC_COMM_PROMPT),
							"Press Start to capture data.");
					}
					
					// Disable everything except stop button and list/plot
					// type while reading
					EnableWindow(GETHCTL(IDC_COMM_CLOSE), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_FILENAME), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_COMM), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_SCALE), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_HELP), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_TEXT), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_OPEN), !bRead);
					EnableWindow(GETHCTL(IDC_COMM_SIZE), !bRead);

					while(bRead)
					{
						// Flush buffers
						PurgeComm(hComm, PURGE_TXABORT | PURGE_RXABORT |
							PURGE_TXCLEAR | PURGE_RXCLEAR);

						// Read data
						// Synchronous read - program blocks until value read
						// Essentially any number of bytes can be read
						// at a time to get a value.  The read operation
						// then times out between values, and then
						// execution continues.
						bSuccess=ReadFile(hComm, szBuffer, BUFFER_SIZE,
							&dwNumBytes, NULL);

						// Put reading into string buffer
						szBuffer[dwNumBytes-2]='\0';	

						// Check if data
						if(dwNumBytes)
						{
							// Increment count
							nNum++;
							nPlot++;

							// Convert data to value
							fData=atof(szBuffer);

							// Scale value
							fData*=fScale;
				
							// Put # into text string for list box
							sprintf(szText, "%d:    ", nNum);

							// Add incoming string
							strcat(szText, szBuffer);

							// Convert scaled data to string
							sprintf(szBuffer, "    %20.15e", fData);
							ShortenExp(szBuffer);

							// Add scaled value to display string
							strcat(szText, szBuffer);

							// A word about data arrays and indices:
							// The plot screen has 8 divisions (portions).
							// nSize determines how many points (seconds)
							// it takes to fill it.
							// For the minimum size nSize=8, it takes 9
							// phase points to define the required 8 line
							// segments, or equivalently, 8 frequency points
							// to define 8 "bars".
							// nNum counts the total # of points, starting at 1.
							// The phase and freq plot data arrays are 0-based.
							// values go into them using the index nPlot
							// which goes from 0 to nSize for phase data
							// and 1 to nSize for frequency data.
							// We use one data point past the end at index
							// [nSize+1] before shifting data.  The arrays
							// are therefore allocated for nSize+2 points.
							
							// Store latest plot points
							// Phase
							fPPoint=fData;

							// Convert phase to freq
							if(nNum>1)
							{
								fFPoint=fData-fPrevData;
							}
							fPrevData=fData;

							// Put data into phase plot array
							// The nNum & nPlot indices were already incremented
							// so the new point goes into index [nPlot-1]
							if(nPlot<=nSize+1) // Are we still filling plot?
							{
								// Add latest phase point 
								fPPlot[nPlot-1]=fPPoint;
							}
							else // nPlot>nSize+1, plot is full
							{
								// Add latest phase point past end
								fPPlot[nPlot-1]=fPPoint;

								// Shift phase plot data to left
								for(i=0; i<nPlot-1; i++)
								{
									fPPlot[i]=fPPlot[i+1];
								}
							}

							// Put data into frequency plot array
							if(nPlot<=nSize+1)  // Are we still filling plot?
							{
								// Add latest freq point
								fFPlot[nPlot-1]=fFPoint;
							}
							else // nPlot>nSize+1, plot is full
							{
								// Add latest freq point past end
								fFPlot[nPlot-1]=fFPoint;
						
								// Shift freq plot data to left
								for(i=0; i<nPlot-1; i++)
								{
									fFPlot[i]=fFPlot[i+1];
								}
							}

							// Display # data points read
							sprintf(szBuffer, "%d", nNum);
							SetWindowText(GETHCTL(IDC_COMM_POINTS), szBuffer);

							// Is main window minimized?
							// Set title accordingly
							if(IsIconic(hwndMain))
							{
								// Put # points into window title
								sprintf(szBuffer, "#=%d", nNum);
								SetWindowText(hwndMain, szBuffer);
							}
							else
							{
								// Use normal title
								SetWindowText(hwndMain,
									"TSC 5110A Communications Program");

								// Plotting on?
								if(bPlot)
								{
									// Update plot
									SendMessage(hDlg, WM_COMMAND,
									GET_WM_COMMAND_MPS(IDC_COMM_PLOTDATA, 0,
									0));
								}
								else
								{
									// Label listing
									SetWindowText(GETHCTL(IDC_COMM_PLOTSCALE),
										"#:  cycles (unscaled), sec (scaled)");
								}
							}								

							// Convert value to string
							sprintf(szData, "%20.15e\r\n", fData);
							ShortenExp(szData);

							// Determine # bytes
							dwNumBytes=strlen(szData);

							// Write data to file
							bSuccess=WriteFile(hFile, szData, dwNumBytes,
								&dwNumBytes, NULL);

							// Prepare for next iteration
							// Must set nPlot=nSize+1 if plot is full
							// It will then be incremented to nSize+2
							// at start of next iteration
							if(nPlot>nSize+1) // Plot full?
							{
								// Set nPlot for next iteration
								nPlot=nSize+1;
							}						
					
						}
						else // Timeout
						{
							strcpy(szText, "Timeout - No Data");
						}

						// Copy text into buffer
						strcpy(szBuffer, szText);

						// Display message
						SendMessage(hDlg, WM_COMMAND,
							GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));
						
						// Check for stop message
						YieldControl();
					} 
					break;
				}
                
                // Detect COM # change
                case IDC_COMM_COMM:
                {
                    if(GET_WM_COMMAND_CMD(wParam, lParam)==CBN_SELCHANGE) 
                    {
                        // Get new COM # that was entered in edit box
                        GetWindowText(GETHCTL(IDC_COMM_COMM), szBuffer,
                            BUFFER_SIZE);

                        // Parse the digit from the COM# string
                        szBuffer[0]=szBuffer[3];
                        szBuffer[1]='\0';
                        nComm=atoi(szBuffer);

                        // Close existing comm port, if any
                        if(bOpen)
                        {
                            // Close the old comm port
                            SendMessage(hDlg, WM_COMMAND, GET_WM_COMMAND_MPS(
                                IDC_COMM_CLOSEPORT, 0, 0));
                        }

                        // Reset the comm open flag
                        bOpen=FALSE;

                        // Reset the comm open indicator
                        Button_SetCheck(GETHCTL(IDC_COMM_OPEN), bOpen);

                        // Save new comm port name
                        GetWindowText(GETHCTL(IDC_COMM_COMM), szPort,
                            BUFFER_SIZE);

                        // Open the new comm port
                        SendMessage(hDlg, WM_COMMAND, GET_WM_COMMAND_MPS(
                            IDC_COMM_OPENPORT, 0, 0));
                    }
                    break;
                }

                // Get filename
                case IDC_COMM_FILENAME:
                {
					// Respond to edit control when it loses the input focus
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (!bErrorTrap) && (GetFocus()!=GETHCTL(IDCANCEL)) &&
						(GetFocus()!=GETHCTL(IDC_COMM_CLOSE)))
					{
                        // Set error flag to avoid multiple error messages
                        bErrorTrap=TRUE;

						// Get filename
						GetWindowText(GETHCTL(IDC_COMM_FILENAME), szText,
							FILENAME_SIZE);

						// Check for valid filename
						// Input is szDataFile to validate
						// Returns TRUE if filename OK, otherwise FALSE
						// Displays messages if filename invalid or if path
						// doesn't exist (offers to create)
						// Also returns file title, path, name and extension
						if(CheckFilename(szText, szTitle, szPath, szName,
							szExt, hDlg, "TSC 5110A Comm"))
						{
							// Accept new filename
							strcpy(szDataFile, szText);
						}
						
						// Redisplay filename - this is done mostly just to
						// return caret to left of edit box to give some
						// user response to enter key
						SetWindowText(GETHCTL(IDC_COMM_FILENAME), szDataFile);
					}
                    else
                    {
                        // Reset error flag
                        bErrorTrap=FALSE;
                    }
					break;
				}

                case IDC_COMM_SCALE:
                {
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the close button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (GetFocus()!=GETHCTL(IDCANCEL)) &&
						(GetFocus()!=GETHCTL(IDC_COMM_CLOSE)))
                    {
                        // Save previous scale value
						fPrev=fScale;
						
						// Get scale factor multiplier
						GetWindowText(GETHCTL(IDC_COMM_SCALE), szBuffer,
							BUFFER_SIZE);
                        fScale=atof(szBuffer);

						// Avoid zero
						if(fScale==0.0)
						{
							fScale=fPrev;
						}

                        // Redisplay scale value
                        sprintf(szBuffer, "%+17.10e", fScale);
						ShortenExp(szBuffer);
                        SetWindowText(GETHCTL(IDC_COMM_SCALE), szBuffer);
					}
                }
                break;

                // Detect size change
                case IDC_COMM_SIZE:
                {
                    // Respond to edit control when it loses the input focus
					// unless it loses it to the close button
                    if((GET_WM_COMMAND_CMD(wParam, lParam)==EN_KILLFOCUS) &&
                        (GetFocus()!=GETHCTL(IDCANCEL)) &&
						(GetFocus()!=GETHCTL(IDC_COMM_CLOSE)))
                    {
                        // Get new size that was entered in edit box
                        GetWindowText(GETHCTL(IDC_COMM_SIZE), szBuffer,
                            BUFFER_SIZE);

						// Convert it to a number that is a multiple of 8
						// because there are 8 plot scale divisions, and we
						// want them to reprresent an even number of seconds.
						// Size must be at least 8.
						nSize=abs(atoi(szBuffer));
						if(nSize<8)
						{
							nSize=8;
						}
						nSize=nSize+(8-(nSize%8)-8);

					// Reallocate memory for plot data arrays
					if((fPPlot=(double*)realloc(fPPlot,
						(nSize+2)*sizeof(F_TYPE)))==NULL)
					{
						// Error message
						MessageBox(hDlg,
							"Unable to allocate memory for plot array.",
							"5110Comm", MB_ICONEXCLAMATION);
					}
					if((fFPlot=(double*)realloc(fFPlot,
						(nSize+2)*sizeof(F_TYPE)))==NULL)
					{
						// Error message
						MessageBox(hDlg,
							"Unable to allocate memory for plot array.",
							"5110Comm", MB_ICONEXCLAMATION);
					}

						// Redisplay size
						sprintf(szBuffer, "%d", nSize);
                        SetWindowText(GETHCTL(IDC_COMM_SIZE), szBuffer);
                      }
                    break;
                }

				// Respond to plot freq radio button
				case IDC_COMM_PLOTFREQ:
				{
					// Set status
					bPlot=TRUE;
					bFreq=TRUE;
					ShowWindow(GETHCTL(IDC_COMM_PLOTAREA), SW_SHOWNORMAL);
					ShowWindow(GETHCTL(IDC_COMM_TEXT), SW_HIDE);

					// Update plot
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_PLOTDATA, 0, 0));
				}
				break;
				
				// Respond to plot phase radio button
				case IDC_COMM_PLOTPHASE:
				{
					// Set status
					bPlot=TRUE;
					bFreq=FALSE;
					ShowWindow(GETHCTL(IDC_COMM_PLOTAREA), SW_SHOWNORMAL);
					ShowWindow(GETHCTL(IDC_COMM_TEXT), SW_HIDE);

					// Update plot
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_PLOTDATA, 0, 0));
				}
				break;

				// Respond to list radio button
				case IDC_COMM_LIST:
				{
					bPlot=FALSE;
					ShowWindow(GETHCTL(IDC_COMM_PLOTAREA), SW_HIDE);
					ShowWindow(GETHCTL(IDC_COMM_TEXT), SW_SHOWNORMAL);

					// Label listing if have data
					if(nNum>0)
					{
						SetWindowText(GETHCTL(IDC_COMM_PLOTSCALE),
							"#:  cycles (unscaled), sec (scaled)");
					}

					// Clear avg freq disp
					SetWindowText(GETHCTL(IDC_COMM_AVG), "");
				}
				break;

				// Respond to CR in an edit control
                // Note: No button is a DEFPUSHBUTTON
                case IDOK:
                {
                    // Does filename edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_COMM_FILENAME))
                    {
                        // Send EN_KILLFOCUS msg to filename edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_COMM_FILENAME, 0,
                            EN_KILLFOCUS));
					}

                    // Does scale factor edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_COMM_SCALE))
                    {
                        // Send EN_KILLFOCUS msg to scale factor edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_COMM_SCALE, 0,
                            EN_KILLFOCUS));
					}

                    // Does size edit control have focus?
                    if(GetFocus()==GETHCTL(IDC_COMM_SIZE))
                    {
                        // Send EN_KILLFOCUS msg to size edit control
                        SendMessage(hDlg, WM_COMMAND,
                            GET_WM_COMMAND_MPS(IDC_COMM_SIZE, 0,
                            EN_KILLFOCUS));
					}
					break;
                }

                // Internal subroutines
                // Internal function to open comm port
                case IDC_COMM_OPENPORT:
                {
                    // Clear the timeouts structure
                    memset(&to, 0, sizeof(to));

                    // Get name of desired comm port
                    GetWindowText(GETHCTL(IDC_COMM_COMM), szBuffer,
                        BUFFER_SIZE);

                    // Open comm port for non-overlapping I/O
                    hComm=CreateFile(szBuffer, GENERIC_READ | GENERIC_WRITE,
                        0, NULL, OPEN_EXISTING, 0, NULL);

                    // Check if comm port opened OK & get current settings
                    // Returns 0 if it fails, non-zero if OK
                    bSuccess=GetCommState(hComm, &dcb);

                    if(bSuccess)
                    {
                        // Set the timeout parameters to make ReadFile()
                        // operate synchronously
						// Set read interval timeout to 50 msec
						// and constant timeout to 2 sec
						// Expect data every second
						// Interval timeout handles end of normal data burst
						// Constant timeout handles loss of data
						// Units are msec
                        to.ReadIntervalTimeout=50;
                        to.ReadTotalTimeoutMultiplier=0;
                        to.ReadTotalTimeoutConstant=2000;
                        to.WriteTotalTimeoutMultiplier=0;
                        to.WriteTotalTimeoutConstant=0;
                        SetCommTimeouts(hComm, &to);

                        // Configure the port
                        dcb.BaudRate=CBR_9600;
                        dcb.fBinary=TRUE;
                        dcb.fParity=FALSE;
                        dcb.fOutxCtsFlow=FALSE;
                        dcb.fOutxDsrFlow=FALSE;
                        dcb.fDtrControl=DTR_CONTROL_DISABLE;
                        dcb.fDsrSensitivity=FALSE;
                        dcb.fTXContinueOnXoff=FALSE;
                        dcb.fOutX=FALSE;
                        dcb.fInX=FALSE;
                        dcb.fErrorChar=FALSE;
                        dcb.fNull=FALSE;
                        dcb.fRtsControl=RTS_CONTROL_DISABLE;
                        dcb.fAbortOnError=FALSE;
                        dcb.XonLim=0;
                        dcb.ByteSize=8;
                        dcb.Parity=NOPARITY;
                        dcb.StopBits=ONESTOPBIT;

                        // Change comm port settings
                        bSuccess=SetCommState(hComm, &dcb);
                    }

                    // Was comm port opened and configured successfully?
                    if(bSuccess)
                    {
                        // Set the comm open flag TRUE
                        bOpen=TRUE;
                    }
                    else // Comm port NG
                    {
                        // Set the comm open flag FALSE
                        bOpen=FALSE;
                    }

                    // Set the comm open indicator
                    Button_SetCheck(GETHCTL(IDC_COMM_OPEN), bOpen);

					// Disable the start button if com port isn't open
					EnableWindow(GETHCTL(IDC_COMM_START), bOpen);
 
					// Put message into listbox
                    if(bOpen)
                    {
                        sprintf(szBuffer, "COM%d Opened", nComm);
                    }
                    else
                    {
                        sprintf(szBuffer, "Could not open COM%d", nComm);
                    }

					// Display message
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));
                    
                    break;
                }

				#if(0) // Original code
                // Internal function to open disk file for overwriting
                case IDC_COMM_OPENFILE:
                {
                    // Open file for writing
					// Old data, if any, is overwritten
					// Read sharing is allowed so data can be read into
					// Stable32 on fly
                    hFile=CreateFile(szDataFile, GENERIC_WRITE,
                        FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);

					if(hFile==INVALID_HANDLE_VALUE)
					{
						// Copy warning message into buffer
						strcpy(szBuffer, "Could not open data file");
					}
					else
					{
						// Copy file opened message into buffer
						strcpy(szBuffer, "Data file opened");
					}

					// Display message
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));
					
					break;
                }
				#endif

				#if(1) // New code 02/18/02
                // Internal function to open disk file for overwriting
                case IDC_COMM_OPENFILE:
                {
					// Check if data filename exists by trying to create it
					// Function will fail if file already exists
					// If so, issue warning that it will be rewritten
                    hFile=CreateFile(szDataFile, GENERIC_WRITE,
                        FILE_SHARE_READ, NULL, CREATE_NEW, 0, NULL);

					if(hFile==INVALID_HANDLE_VALUE)  // File exists
					{
						// Display message box
						nDummy=MessageBox(hwndMain,
							"A data file with this name already exists.\n"
							"Do you want to append data to it?\n\n"
							"Yes:\tAppend data\n"
							"No:\tOverwrite data\n"
							"Cancel:\tChoose another filename",
							"5110Comm - Append File?",
							MB_YESNOCANCEL | MB_ICONQUESTION | MB_DEFBUTTON3);
						
						// Check for "Cancel" response to stop file open operation
						if(nDummy==IDCANCEL)
						{
							// Change the stop button to start
							SetWindowText(GETHCTL(IDC_COMM_START), "&Start");

							// Change the prompt text
							SetWindowText(GETHCTL(IDC_COMM_PROMPT),
								"Press Start to read data via comm port.");

							// Stop
							bRead=FALSE;

							// Give filename edit box the input focus
							SetFocus(GETHCTL(IDC_COMM_FILENAME));

							break;
						}
					}
					else // File didn't exist - new one was created
					{
						nDummy=IDCANCEL;  // Flag that new file was created
					}

					// Open file for writing
					// Read sharing is allowed so data can be read into
					// Stable32 on fly

					// Was response to append question "No"?
					if(nDummy==IDNO)
					{
						// Old data, if any, is overwritten
						hFile=CreateFile(szDataFile, GENERIC_WRITE,
							FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
					}

					// Was response to append question "Yes"?
					if(nDummy==IDYES)
					{
						// Old data, if any, is appended to
						hFile=CreateFile(szDataFile, GENERIC_WRITE,
							FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);

						// Position file pointer to end of file
						SetFilePointer( hFile, 0, 0, FILE_END);
					
					}

					if(hFile==INVALID_HANDLE_VALUE)
					{
						// Copy warning message into buffer
						strcpy(szBuffer, "Could not open data file");
					}
					else // Copy file opened message into buffer
					{
						if(nDummy==IDCANCEL) // New file
						{
							//
							strcpy(szBuffer, "Data file created");
						}

						if(nDummy==IDNO) // Overwrite file
						{
							//
							strcpy(szBuffer, "Data file overwritten");
						}

						if(nDummy==IDYES) // Append file
						{
							//
							strcpy(szBuffer, "Data file appended");
						}
					}

					// Display message
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));
					
					break;
                }
				#endif
                
				// Internal function to close comm port
                case IDC_COMM_CLOSEPORT:
                {
                    // Close comm port
                    bSuccess=CloseHandle(hComm);

                    // Check if comm port closed OK
                    if(bSuccess)
                    {
                        sprintf(szBuffer, "%s Closed", szPort);
                    }
                    else
                    {
                        sprintf(szBuffer, "Could not close %s", szPort);
                    }
                    
					// Display message
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_MESSAGE, 0, 0));

                    break;
                }
                
                // Internal function to go to bottom of listbox
                case IDC_COMM_LBBOTTOM:
                {
                    // Get # items in list box
                    i=SendMessage(GETHCTL(IDC_COMM_TEXT), LB_GETCOUNT,
                        0, 0);

                    // Go to last item
                    SendMessage(GETHCTL(IDC_COMM_TEXT), LB_SETCURSEL,
                        (WPARAM)i-1, 0);

					// Eliminate selection hightlight
					SendMessage(GETHCTL(IDC_COMM_TEXT), LB_SETCURSEL,
                        (WPARAM)-1, 0);

                    break;
                }

                // Internal function to write message in list box
				// Message to be written is in szBuffer
                case IDC_COMM_MESSAGE:
                {
                    // Write message
                    ListBox_AddString(GETHCTL(IDC_COMM_TEXT), szBuffer);

                    // Find out how many items there are in the list box
                    i=ListBox_GetCount(GETHCTL(IDC_COMM_TEXT));

                    // Remove first item if more than allowed
                    // This is done because of the possibility of many items
                    // during status monitoring
                    if(i>nSize)
                    {
                        ListBox_DeleteString(GETHCTL(IDC_COMM_TEXT), 0);
                    }

                    // Go to bottom of message listbox
                    SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_LBBOTTOM, 0, 0));
         
                    break;
                }

				// Internal function to plot phase or freq data
				// Plot updated each meas cycle
				// Current # phase plot points = nPlot
				// Data fills plot, then scrolls to show only latest points
				// nPlot starts at 1 when 1st point is read
				// and grows to and remains at nSize
				// Plot y-scale is autoscaled
				case IDC_COMM_PLOTDATA:
				{
					// Find avg freq
					// Freq data starts at i=1
					fAvg=0.0;
					for(i=1; i<nPlot; i++)
					{
						fAvg+=fFPlot[i];
					}
					fAvg/=(double)(nPlot-1);

					// Find std dev of freq (when nPlot>2)
					if(nPlot>2)
					{
						fDev=0.0;
						for(i=1; i<nPlot; i++)
						{
							fDev+=((fFPlot[i]-fAvg)*(fFPlot[i]-fAvg));
						}
						fDev=sqrt(fDev)/(double)(nPlot-1);
					}

					// Set y-scale
                    if(bFreq)
					{
						// Find range
						fFMax=fFPlot[1];
						fFMin=fFPlot[1];
						for(i=1; i<nPlot; i++)
						{
							if(fFPlot[i]<fFMin)
							{
								fFMin=fFPlot[i];
							}
							if(fFPlot[i]>fFMax)
							{
								fFMax=fFPlot[i];
							}
						}
						fRange=fFMax-fFMin;

					}
					else
					{
						// Find range
						fPMax=fPPlot[0];
						fPMin=fPPlot[0];
						for(i=1; i<nPlot; i++)
						{
							if(fPPlot[i]<fPMin)
							{
								fPMin=fPPlot[i];
							}
							if(fPPlot[i]>fPMax)
							{
								fPMax=fPPlot[i];
							}
						}
						fRange=fPMax-fPMin;
					}

					// Label scale factors
					if(nNum>1)
					{
						nDiv=(int)(nSize/X_DIVS);
						
						if(bFreq)
						{
							sprintf(szBuffer,
								"X-Axis: %d sec/div, Y-Axis: %5.3e to %5.3e = %5.3e/div",
								nDiv, fFMin, fFMax, fRange/Y_DIVS); 
							ShortenExp(szBuffer);
							SetWindowText(GETHCTL(IDC_COMM_PLOTSCALE), szBuffer);

						}
						else // Phase
						{
							sprintf(szBuffer,
								"X-Axis: %d sec/div, Y-Axis: %7.5e to %7.5e = %5.3e/div",
								nDiv, fPMin, fPMax, fRange/Y_DIVS);
							ShortenExp(szBuffer);
							SetWindowText(GETHCTL(IDC_COMM_PLOTSCALE),
								szBuffer);
						}

						// Show last freq, avg freq and its std err
						sprintf(szBuffer, "Last Freq= %8.5e; Avg Freq= %8.5e +/- %6.3e (scaled)",
							fFPlot[nPlot-1], fAvg, fDev); 
						ShortenExp(szBuffer);
						SetWindowText(GETHCTL(IDC_COMM_AVG), szBuffer);
					}

					// Draw plot
					SendMessage(hDlg, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDC_COMM_BITMAPPLOT, 0, 0));

					// Done
					break;
				}
				
				// Internal function to plot phase or freq data
				// Plot onto a bitmap in memory, which is then transferred
				// to the screen all at once without flicker
				case IDC_COMM_BITMAPPLOT:
				{
					// Create memory device context
					hdcMem=CreateCompatibleDC(hdcPlot);
                    
					// Get size of plot area
					GetClientRect(hPlot, &rect);

					// Create bitmap of this size
					hBitmap=CreateCompatibleBitmap(hdcPlot,
						(rect.right-rect.left), (rect.bottom-rect.top));

					// Select bitmap into device context
					SelectObject(hdcMem, hBitmap);

					// Now we can draw on this memory device context
					// Make plot background white
					FillRect(hdcMem, &rect, GetStockObject(WHITE_BRUSH));

					// Check for zero range
                    if(fRange)
                    {
						// Set scales
						// The -1 avoids stray overrange pixels at bottom
						fYScale=(double)(rect.bottom-rect.top-1);
			            fXScale=(double)(rect.right-rect.left-1);

                        // Select black pen
                        SelectObject(hdcMem, GetStockObject(BLACK_PEN));

						// Draw horizontal (Y-Axis) grid = Y_DIVS divs 
                        for(i=1; i<Y_DIVS; i++)
						{
                            MoveToEx(hdcMem, 0, (int)((i/Y_DIVS)*fYScale),
                                NULL);
                            LineTo(hdcMem, (int)fXScale,
                                (int)((i/Y_DIVS)*fYScale));
                        }

						// Draw vertical (X-Axis) grid = X_DIVS divs
                        for(i=1; i<X_DIVS; i++)
                        {
                            MoveToEx(hdcMem, (int)((i/X_DIVS)*fXScale), 0,
								NULL);
                            LineTo(hdcMem, (int)((i/X_DIVS)*fXScale),
                                (int)fYScale);
                        }

						// Select red pen
						SelectObject(hdcMem, hRedPen);

						if(bFreq)
						{
							// Plot freq data points
							for(i=0; i<nPlot-2; i++)
							{
								// Plot freq data points as freq data with
								// horizontal lines
								// Move to current point
								MoveToEx(hdcMem,
									(int)(((double)i/(double)(nSize))*
									fXScale),
									(int)(((fFMax-fFPlot[i+1])/(fRange))*
									(fYScale)), NULL);

								// Draw horiz line from there to next point
								LineTo(hdcMem,
									(int)(((double)(i+1)/
									(double)(nSize))*fXScale),
									(int)(((fFMax-fFPlot[i+1])/
									(fRange))*(fYScale)));

								if(i<nPlot-3) // Don't draw final vertical line
								{
									// Draw vert line from there to next point
									nLastY=(int)(((fFMax-fFPlot[i+2])/
										(fRange))*(fYScale));
									LineTo(hdcMem, (int)(((double)(i+1)/
										(double)(nSize))*
										fXScale), nLastY);
								}

								// Select green pen
								SelectObject(hdcMem, hGreenPen);

								// Draw line at average
								MoveToEx(hdcMem, 0,
									(int)(((fFMax-fAvg)/fRange)*fYScale),
									NULL);
								LineTo(hdcMem,
									(int)((((double)(nPlot-2))/
									((double)(nSize)))*(fXScale)),
									(int)(((fFMax-fAvg)/fRange)*fYScale));

								// Reselect red pen
								SelectObject(hdcMem, hRedPen);
							}
						}
						else // Phase
						{
							// Plot phase data points
							for(i=0; i<nPlot-1; i++)
							{
								// Move to current point
								MoveToEx(hdcMem,
									(int)(((double)i/(double)(nSize))*
									fXScale),
									(int)(((fPMax-fPPlot[i])/(fRange))*
									(fYScale)),
									NULL);

								// Draw line from there to next point
								LineTo(hdcMem,
									(int)(((double)(i+1)/
									(double)(nSize))*fXScale),
									(int)(((fPMax-fPPlot[i+1])/
									(fRange))*(fYScale)));
							}
						}
						
						// Delete memory device context
						DeleteDC(hdcMem);
						
						// Draw the bitmap in the plot area
						DrawBitmap(hdcPlot, hBitmap, 0, 0);

						// Delete bitmap
						DeleteObject(hBitmap);
					}
                    else
                    {
						// Display error message instead of plot
						SetWindowText(GETHCTL(IDC_COMM_PLOTAREA),
							"No Plot Data");
					}
					break;
				}
                
				// Cancel - the X button at the top
                case IDCANCEL:
				{
					// Ignore cancel if reading data
					// Must push stop button first
					if(bRead)
					{
						break;
					}
				}

				// Fall into close
				// Close function
                case IDC_COMM_CLOSE:
                {
                    // Close open comm port, if any
                    if(bOpen)
                    {
                        // Close comm port
                        SendMessage(hDlg, WM_COMMAND, GET_WM_COMMAND_MPS(
                            IDC_COMM_CLOSEPORT, 0, 0));
                    }

					// Save Comm port # in ini file
					sprintf(szBuffer, "%d", nComm);
					WritePrivateProfileString("Preferences", "Comm", szBuffer,
						szIniFile);

					// Save plot flag in ini file
					sprintf(szBuffer, "%d", bPlot);
					WritePrivateProfileString("Preferences", "Plot", szBuffer,
						szIniFile);

					// Save freq flag in ini file
					sprintf(szBuffer, "%d", bFreq);
					WritePrivateProfileString("Preferences", "Freq", szBuffer,
						szIniFile);

					// Save plot & list size in ini file
					sprintf(szBuffer, "%d", nSize);
					WritePrivateProfileString("Preferences", "Size",
						szBuffer, szIniFile);

					// Save scale factor in ini file
					sprintf(szBuffer, "%e", fScale);
					ShortenExp(szBuffer);
					WritePrivateProfileString("Preferences", "Scale", szBuffer,
						szIniFile);

					// Save data filename in ini file
					WritePrivateProfileString("Preferences", "File",
						szDataFile,	szIniFile);

                    // Close the comm port
					CloseHandle(hComm);
					
					// Close file
					if(hFile)
					{
						CloseHandle(hFile);
					}

					// Delete plot dc
					ReleaseDC(hwndMain, hdcPlot);

					// Free plot data arrays
					free(fPPlot);
					free(fFPlot);

                    // Select black pen
                    SelectObject(hdcPlot, GetStockObject(BLACK_PEN));

					// Delete red & green pens
					DeleteObject(hRedPen);
					DeleteObject(hGreenPen);

					// Destroy main window
					DestroyWindow(hwndMain);

					// Close program
                    PostQuitMessage(0);
					return 0;
               }
                return TRUE;
            }
        }
        break;
    }
    return FALSE;
}
