/*****************************************************************************/
/*                                                                           */
/*   MJDate3.c                                                               */
/*                                                                           */
/*   Date functions for MJDate program                                       */
/*                                                                           */
/*   (c) 1999  W. Riley  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

// Includes
#include "MJDate.h"

/*****************************************************************************/
/*                                                                           */
/*      						BreakDate()                                  */
/*                                                                           */
/*		Function to break down a Gregorian date into month, day and year     */
/*                                                                           */
/* 		Parameters:	LONG	gdate	Gregorian date (yyyymmdd)       	     */
/*      	        INT *	mo		month (1-12)                             */
/*                  INT *	da		day   (1-31)                             */
/*                  INT *	yr		year  (4-digit)                          */
/*                                                                           */
/*      Return:     void			None                                  	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from brkdate() of Hammer Library              */
/*						Modified code so full 4-digit year is returned       */
/*                                                                           */
/*****************************************************************************/

void WINAPI BreakDate(LONG gdate, INT *mo,INT *da,INT *yr)
{
	*yr=(int) (gdate/10000L);
	*mo=(int) ((gdate/100L) % 100L);
	*da=(int) (gdate%100L);
}

/*****************************************************************************/
/*                                                                           */
/*      						DayOfWeek()                                  */
/*                                                                           */
/*		Function to find the day of the week for a given Gregorian date      */
/*                                                                           */
/*		Note: Calls Brkdate() above                                          */
/*                                                                           */
/* 		Parameters:	LONG	gdate	Gregorian date (yyyymmdd)       	     */
/*                                                                           */
/*      Return:     INT 			Day of week (0=Sunday, 6=Saturday)   	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from dayofwk() of Hammer Library              */
/*						Changed error return to -1                           */
/*						Modified for 4-digit year from BreakDate()           */
/*                                                                           */
/*****************************************************************************/

INT WINAPI DayOfWeek(LONG gdate)
{
	// Local variables
	int a, b, mo, da, yr;
	static int mos[12] = { 0,3,3,6,1,4,6,2,5,0,3,5 };

	BreakDate(gdate, &mo, &da, &yr);

	// Convert 4-digit year into 2-digit year
	// yr=yr % 100;  // Only OK 1900-1999
	yr=yr-1900;      // Better; OK 1900-2099

	if ( mo<1 || mo>12 || da<1 || da>31 || yr<0 )
	{
		return (-1);	/* error - bad date */
	}

	a = (yr/4) + yr + da + mos[mo-1];
	b = (a % 7) + 1;

	if (mo<=2 && yr!=0 && (yr % 4)==0)
	{
		b = (b==0 ? 6 : b) - 1;
	}

	b = (b==0 ? 7 : b) - 1;

	return (b);	/* return day of week (0-6) */
}

/*****************************************************************************/
/*                                                                           */
/*      						DateToJulian()                               */
/*                                                                           */
/*		Function to convert Gregorian date to Julian day number              */
/*                                                                           */
/* 		Parameters:	LONG	gdate	Gregorian date (yyyymmdd)       	     */
/*                                                                           */
/*      Return:     LONG 	jdate	Julian day number                   	 */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from datjul() of Hammer Library               */
/*						Changed error return to -1                           */
/*						Modofoed to use 4-digit year return from BreakDate() */
/*                                                                           */
/*****************************************************************************/

LONG WINAPI DateToJulian(LONG gdate)
{
	// Local variables
	LONG jdate;	/* Julian Day Number for given date */
	INT mo, da, yr, c, ya;

	if (gdate<=0L || gdate==19000000L)	/* if zero or negative, return -1 */
	{
		return (-1L); /* error */
	}

	BreakDate(gdate, &mo, &da, &yr);

	if (mo>2)
	{
		mo-=3;
	}
	else
	{
		mo+=9;
		yr--;
	}

	c=yr/100;
	ya=yr%100;
	jdate=(146097L*c)/4L + (1461L*ya)/4L + (153L*mo+2L)/5L + da + 1721119L;

	return (jdate);
}

/*****************************************************************************/
/*                                                                           */
/*      						JulianToDate()                               */
/*                                                                           */
/*		Function to convert Julian day number to Gregorian date              */
/*                                                                           */
/* 		Parameters:	LONG	jdate	Julian day number       	             */
/*                                                                           */
/*      Return:     LONG   			Gregorian date (yyyymmdd)                */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from juldat() of Hammer Library               */
/*						Changed error code to -1                             */
/*                                                                           */
/*****************************************************************************/

LONG WINAPI JulianToDate(LONG jdate)
{
	extern long int makdate();
	long int gdate;	/* Gregorian Date (yyyymmdd) for given Julian Day */
	int mo, da, yr;
	long int td, tm, ty;

	if (jdate<=0L)	/* if zero or negative, return -1 */
	{
		return (-1L);
	}

	jdate-=1721119L;
	ty=(4L*jdate-1L)/146097L;
	jdate=4L*jdate - 1L - 146097L*ty;
	td=jdate/4L;

	jdate=(4L*td+3)/1461L;
	td=4L*td + 3L - 1461L*jdate;
	td=(td+4L)/4L;

	tm=(5L*td - 3L)/153L;
	td=5L*td - 3L - 153L*tm;
	td=(td+5L)/5L;

	ty=100L*ty + jdate;
	if (tm<10L)
	{
		tm+=3L;
	}
	else
	{
		tm-=9L, ty++;
	}

	da=(int) td; mo=(int) tm; yr=(int) ty;
	gdate=MakeDate(mo, da, yr);

	return(gdate);
}

/*****************************************************************************/
/*                                                                           */
/*      						MakeDate()                                   */
/*                                                                           */
/*		Function to make Gregorian date from month, day and year             */
/*                                                                           */
/* 		Parameters:	INT		mo		Month (1-12)            	             */
/*                  INT		da		Day (1-31)                               */
/*                  INT		yr      Year (2 or 4-digit)                      */
/*                                                                           */
/*      Return:     LONG   			Gregorian date (yyyymmdd)                */
/*                                                                           */
/*		Revision record:                                                     */
/*      	07/17/97	Cloned from makdate() of Hammer Library              */
/*                                                                           */
/*****************************************************************************/

LONG WINAPI MakeDate(INT mo, INT da, INT yr)
{
	yr = (yr<=100 ? yr+1900 : yr);
	return ((long int) yr*10000L + mo*100L + da);
}

/*****************************************************************************/
/*                                                                           */
/*      						MJDtoGPS()                                   */
/*                                                                           */
/*		Function to convert MJD to GPS Week #                                */
/*                                                                           */
/* 		Parameters:	LONG	MJD                               	             */
/*                                                                           */
/*      Return:     INT   	GPS Week #                                       */
/*                                                                           */
/*		Note:	The origin (time zero) of GPS System Time was 00:00:00 UTC   */
/*				6 January 1980, Julian Day 2,444,244.500, MJD 44244.  A      */
/*				GPS Cycle is 1024 weeks.                                     */
/*                                                                           */
/*		Revision record:                                                     */
/*      	01/25/98	Created                                              */
/*                                                                           */
/*****************************************************************************/

INT WINAPI MJDtoGPS(LONG MJD)
{
	return ((int)((MJD-44244L)/7)%1024);
}

/*****************************************************************************/
/*                                                                           */
/*                           DateConv()                                      */
/*                                                                           */
/*      Function to convert month & year to day of week, DOY and MJD,        */
/*      plus # days in month.                                                */
/*                                                                           */
/*      Calling program sets month and year.  1st day of month is assumed.   */
/*      Function sets corresponding day of week, day of year, MJD and        */
/*      # days in month.                                                     */
/*                                                                           */
/*      Uses date functions from Hammer library.                             */
/*                                                                           */
/*      Parameters:                                                          */
/*          int  lpnMonth          pointer to month (January=1)              */
/*          int  lpnDaysInMonth    pointer to # days in month                */
/*          int  lpnYear           pointer to year (e.g. 1995)               */
/*          int  lpnDOW            pointer to day of week (Sunday=0)         */
/*          int  lpnDOY            pointer to day of year                    */
/*          long lplnMJD           pointer to Modified Julian Date           */
/*                                                                           */
/*      Returns nothing (void)                                               */
/*                                                                           */
/*      Revision record:                                                     */
/*          03/30/96    Adapted from date_conv() of SCRN_CAL.C               */
/*          04/08/97    Fixed bug in century leap year logic                 */
/*			07/18/97	Changed to new date function to avoid Hammer Library */
/*                                                                           */
/*****************************************************************************/

void WINAPI DateConv(int *lpnMonth, int *lpnDaysInMonth,
	int *lpnYear, int *lpnDOW, int *lpnDOY, long *lplnMJD)
{
	static int sum[] = { 0, 31, 59, 90, 120, 151, 181, 212,
						  243, 273, 304, 334, 365 };
	static int num_days[] = {31, 28, 31, 30, 31, 30, 31, 31,
						  30, 31, 30, 31};
	// LOCAL VARIABLES
	int month;
	int year;
	int dow;                         /* day of week: 0=Sunday ... 6=Saturday */
	int day=1;                                      /* day of month always 1 */
	int doy;                                                  /* day of year */
	int leap=0;                                            /* leap year flag */
	long int gdate;                                        /* Gregorian date */
	long int jdate;                                          /* Julian day # */
	long int mjd;                                     /* Modified Julian Day */

	// INITIALIZATIONS
	month=*lpnMonth;
	year=*lpnYear;

	// DATE CONVERSIONS
	gdate=MakeDate(month, day, year);
	BreakDate(gdate, &month, &day, &year);
	dow=DayOfWeek(gdate);

	doy=sum[month-1]+day;

	if( year % 4 == 0 && year % 100 != 0 || year % 400 == 0 )
	{
		leap=1;
		if(month>2)
		{
			doy++;
		}                                                          /* end if */
	}                                                              /* end if */

	jdate=DateToJulian(gdate);
	mjd=jdate-2400001L;

	// PASS RESULTS
	*lplnMJD=mjd;
	*lpnDOW=dow;
	*lpnDOY=doy;

	if(month!=2)
	{
		leap=0;
	}                                                              /* end if */

	*lpnDaysInMonth=num_days[month-1]+leap;
}                                                      /* end date_convert() */

/*****************************************************************************/
/*                                                                           */
/*                           DrawBitmap()                                    */
/*                                                                           */
/* 	Function to draw a bitmap on a display surface.                          */
/*                                                                           */
/*  See Petzold, "Programming Windows 3.1, p.631                             */
/*                                                                           */
/*****************************************************************************/

void WINAPI DrawBitmap(HDC hdc, HBITMAP hBitmap, short xStart,
	short yStart)
{
	BITMAP bm;
	HDC hdcMem;
	POINT ptSize, ptOrg;

	hdcMem=CreateCompatibleDC(hdc);
	SelectObject(hdcMem, hBitmap);
	SetMapMode(hdcMem, GetMapMode(hdc));

	GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
	ptSize.x=bm.bmWidth;
	ptSize.y=bm.bmHeight;
	DPtoLP(hdc, &ptSize, 1);

	ptOrg.x=0;
	ptOrg.y=0;
	DPtoLP(hdcMem, &ptOrg, 1);

	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y,
		hdcMem, ptOrg.x, ptOrg.y, SRCCOPY);

	DeleteDC(hdcMem);
}
/*****************************************************************************/
