// Units.c
// Program to parse data file and apply units to values
// Each line comprises: numeric value, whitespace, and (optionally) certain units
// Lines starting with a non-numeric character are ignored
// Allowable units are: Gs, Ms, ks, s, ms, us, ns, ps and fs for phase
// and GHz, MHz, kHz, Hz, mHz, uHz, nHz, pHz and fHz for frequency
// Scaling factors are: 10^9, 10^6, 10^3, 10^0, 10^-3, 10^-6, 10^-9,
// 10^-12 and 10^-15 respectively
// Lines are parsed individually for units
// Input file is 1st cmd line arg
// Output file is 2nd cmd line arg
// 3rd cmd line arg is optional nominal freq for fractional freq scaling
// (c) W. Riley Hamilton Technical Services 06/04/11 All Rights Reserved
// 06/05/11: Added more units, fractional freq scaling, help and user interface

// Includes
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

//Defines
#define BOOL int
#define TRUE 1
#define FALSE 0
#define BUFFER_SIZE 80
#define FILENAME_SIZE 128

int main(int argc, char *argv[])
{
	// Local variables
	// Chars
	char szLine[BUFFER_SIZE+1];		// Input buffer
	char szIn[FILENAME_SIZE+1];		// Input filename
	char szOut[FILENAME_SIZE+1];	// Output filename
	char szNom[BUFFER_SIZE+1];		// Nominal frequency as string
	// Ints
	int n;							// # lines
	//Bools
	BOOL bClose=FALSE;				// Flag to close program
	// Doubles
	double fValue;					// Data value
	double fScale;					// Scale factor
	double fNom=0.0;					// Nominal frequency as double
	// Misc
	FILE *fIn;           			// DOS file handle for input
	FILE *fOut;           			// DOS file handle for output

	// Get filenames as 1st 2 args.  If no args, show help
	// Show usage if no args
	// Shpw help if arg is ?

	// Are there any command line arguments?
	if(argc>1)
	{
		if(argv[1][0]=='?')
		{
			printf("\nUnits program to process a phase or frequency data file with units.\n");
			printf("It can process data with units from a counter or other instrument\n");
			printf("in the form of lines like 1.23456789 ns for analysis by Stable32.\n");
			printf("Each line comprises numeric value, whitespace and optional units.\n");
			printf("No scaling is done for lines without units.\n");
			printf("Lines starting with a non-numeric character are ignored,\n");
			printf("and numeric lines are parsewd individually for their units.\n");
			printf("Units are Gs, Ms, ks, s, ms, us, ns, ps and fs for phase data,\n");
			printf("and GHz, MHz, kHz, Hz, mHz, uHz, nHz, pHz and fHz for frequency data.\n");
			printf("Scaling factors are: 10^9, 10^6, 10^3, 10^0, 10^-3, 10^-6, 10^-9,\n");
			printf("10^-12 and 10^-15 respectively.\n");
			printf("Program usage is: units infile outfile [nomfreq]\n");
			printf("where infile and outfile are required input and output data filenames,\n");
			printf("and nomfreq is optional nominal frequency for fractional frequency scaling.\n");
			printf("Output is a single column of scaled data values in exponential format.\n");
			bClose=TRUE;
		}
		else
		{
			if(argc==2)
			{
				printf("\nToo few arguments.\n");	
				printf("Usage: units infile outfile [nomfreq] or units ? for help\n");
				bClose=TRUE;
			}
			else if(argc==3) // Input and output filenames
			{
				strcpy(szIn, argv[1]);
				strcpy(szOut, argv[2]);
			}
			else if(argc==4) // Input and output filenames plus nominal frequency
			{
				strcpy(szIn, argv[1]);
				strcpy(szOut, argv[2]);
				strcpy(szNom, argv[3]);
				fNom=atof(szNom);
			}
		}
	}
	else // No args
	{
		printf("Usage: units infile outfile [nomfreq] or units ? for help\n");
		bClose=TRUE;
	}

	if(bClose)
	{
		return 1; // Close program
	}

	// Open existing file for reading
	if((fIn=fopen(szIn, "r"))==NULL)
	{
		// Error - could not open input file
		printf("\nCouldn't open input file %s\n", szIn);
		return 1;
	}

	// Open new file for writing
	if((fOut=fopen(szOut, "w"))==NULL)
	{
		// Error - could not open output file
		printf("\nCouldn't open output file %s\n", szOut);
		return 1;
	}

	// Print starting message
	printf("\nProcessing file %s to scale data per units to %s\n", szIn, szOut);

	// Initialize # lines
	n=0;

	// Read and process lines from input file
	while((fgets(szLine, BUFFER_SIZE, fIn) != NULL))
	{
		// Ignore non-numeric lines
		// Reject lines starting with a-z or A-Z
		if(isalpha(szLine[0]))
		{
			continue;
		}

		// Also want to reject lines with punctuation symbol
		// but allowing + and - as special cases
		if(((ispunct(szLine[0])) && (szLine[0]!='+')) && (szLine[0]!='-'))
		{
			continue;
		}

		// Parse value
		sscanf(szLine, "%le", &fValue);

		// Parse units
		// Initialize scale factor
		fScale=1.0;
		
		// Look for Gs or GHz
		if((strstr(szLine, "Gs")!=NULL) || (strstr(szLine, "GHz")!=NULL))
		{
			fScale=1e9;
		}

		// Look for Ms or MHz
		if((strstr(szLine, "Ms")!=NULL) || (strstr(szLine, "MHz")!=NULL))
		{
			fScale=1e6;
		}

		// Look for ks or kHz
		if((strstr(szLine, "ks")!=NULL) || (strstr(szLine, "kHz")!=NULL))
		{
			fScale=1e3;
		}

		// Look for s or Hz
		if((strstr(szLine, "s")!=NULL) || (strstr(szLine, "Hz")!=NULL))
		{
			fScale=1.0;
		}

		// Look for ms or mHz
		if((strstr(szLine, "ms")!=NULL) || (strstr(szLine, "mHz")!=NULL))
		{
			fScale=1e-3;
		}

		// Look for us or uHz
		if((strstr(szLine, "us")!=NULL) || (strstr(szLine, "uHz")!=NULL))
		{
			fScale=1e-6;
		}

		// Look for ns or nHz
		if((strstr(szLine, "ns")!=NULL) || (strstr(szLine, "nHz")!=NULL))
		{
			fScale=1e-9;
		}

		// Look for ps or pHz
		if((strstr(szLine, "ps")!=NULL) || (strstr(szLine, "pHz")!=NULL))
		{
			fScale=1e-12;
		}

		// Look for fs or fHz
		if((strstr(szLine, "fs")!=NULL) || (strstr(szLine, "fHz")!=NULL))
		{
			fScale=1e-15;
		}

		// Apply scaling factor
		fValue*=fScale;

		// Apply fractional frequency scaling if fNom
		if(fNom)
		{
			fValue=(fValue-fNom)/fNom;
		}

		// Compose output line
		sprintf(szLine, "%16.14e\n", fValue);

		// Write line to output file
		fputs(szLine, fOut);

		// Increment # lines
		n++;
	}

	// Close files
	fclose(fIn);
	fclose(fOut);

	// Close program
	printf("%d Lines processed\n", n);
	return 0;
}

